﻿using System;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;

namespace CustomZoomLevels
{
    public partial class TestForm : Form
    {
        private MapEngine mapEngine = new MapEngine();
        private Bitmap bitmap = null;
       
       public TestForm()
        {
            InitializeComponent();
        }

        private void TestForm_Load(object sender, EventArgs e)
        {
           

            ShapeFileFeatureLayer layer1 = new ShapeFileFeatureLayer(@"..\..\Data\austinstreets.shp");
            LineStyle lineStyle = new LineStyle(new GeoPen(GeoColor.StandardColors.Red, 2));
            //Sets the custom zoom levels based on a scale of 1: 150,000
            layer1.ZoomLevelSet = GetCustomZoomLevelSet(150000);
            //Sets the symbols for each ten custom zoom levels.
            layer1.ZoomLevelSet.CustomZoomLevels[0].DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.Red, 2, true);
            layer1.ZoomLevelSet.CustomZoomLevels[1].DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.Green, 2, true);
            layer1.ZoomLevelSet.CustomZoomLevels[2].DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.Orange, 2, true);
            layer1.ZoomLevelSet.CustomZoomLevels[3].DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.OrangeRed, 2, true);
            layer1.ZoomLevelSet.CustomZoomLevels[4].DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.Blue, 2, true);
            layer1.ZoomLevelSet.CustomZoomLevels[5].DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.BlueViolet, 2, true);
            layer1.ZoomLevelSet.CustomZoomLevels[6].DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.DarkSeaGreen, 2, true);
            layer1.ZoomLevelSet.CustomZoomLevels[7].DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.DeepPink, 2, true);
            layer1.ZoomLevelSet.CustomZoomLevels[8].DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.YellowGreen, 2, true);
            layer1.ZoomLevelSet.CustomZoomLevels[9].DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.Violet, 2, true);

            mapEngine.StaticLayers.Add(layer1);

            // Set the extent and the background color
            mapEngine.BackgroundFillBrush = new GeoSolidBrush(GeoColor.StandardColors.LightGoldenrodYellow);
            layer1.Open();
            mapEngine.CurrentExtent = ExtentHelper.GetDrawingExtent(layer1.GetBoundingBox(), Map.Width, Map.Height);
            layer1.Close();

            DrawImage();
        }

        //This function sets ten zoom levels based on a reference scale. Each zoom level is half in scale of the previous one.
        private ZoomLevelSet GetCustomZoomLevelSet(double initialScale)
        {
            ZoomLevelSet zoomLevelSet = new ZoomLevelSet();
            zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(initialScale));
            zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(initialScale / 2));
            zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(initialScale / 4));
            zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(initialScale / 8));
            zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(initialScale / 16));
            zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(initialScale / 32));
            zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(initialScale / 64));
            zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(initialScale / 128));
            zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(initialScale / 256));
            zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(initialScale / 512));
            return zoomLevelSet;
        } 

    
        private void DrawImage()
        {
            if (bitmap != null) { bitmap.Dispose(); }
            bitmap = new Bitmap(Map.Width, Map.Height);
            mapEngine.OpenAllLayers();
            mapEngine.DrawStaticLayers(bitmap, GeographyUnit.DecimalDegree);
            mapEngine.DrawDynamicLayers(bitmap, GeographyUnit.DecimalDegree);
            mapEngine.CloseAllLayers();

            Map.Image = bitmap;
        }

        private void ToolBar_ButtonClick(object sender, ToolBarButtonClickEventArgs e)
        {
            switch (e.Button.Tag.ToString())
            {
                case "Zoom In":
                    mapEngine.CurrentExtent.ScaleDown(50);
                    break;
                case "Zoom Out":
                    mapEngine.CurrentExtent.ScaleUp(50);
                    break;
                case "Full Extent":
                    mapEngine.CurrentExtent = ExtentHelper.GetDrawingExtent(new RectangleShape(-180.0, 83.0, 180.0, -90.0), Map.Width, Map.Height);
                    break;
                case "Pan Left":
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Left, 20);
                    break;
                case "Pan Right":
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Right, 20);
                    break;
                case "Pan Up":
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Up, 20);
                    break;
                case "Pan Down":
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Down, 20);
                    break;
                default:
                    break;
            }
            DrawImage();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void Map_MouseMove(object sender, MouseEventArgs e)
        {
            //Displays the X and Y in screen coordinates.
            statusStrip1.Items["toolStripStatusLabelScreen"].Text = "X:" + e.X + " Y:" + e.Y;

            //Gets the PointShape in world coordinates from screen coordinates.
            PointShape pointShape = ExtentHelper.ToWorldCoordinate(mapEngine.CurrentExtent, new ScreenPointF(e.X, e.Y), Map.Width, Map.Height);

            //Displays world coordinates.
            statusStrip1.Items["toolStripStatusLabelWorld"].Text = "(world) X:" + Math.Round(pointShape.X, 4) + " Y:" + Math.Round(pointShape.Y, 4);
        }
    }
}
